/*
 *
 *    Artistic License
 *
 *    Preamble
 *
 *    The intent of this document is to state the conditions under which a
 *    Package may be copied, such that the Copyright Holder maintains some
 *    semblance of artistic control over the development of the package,
 *    while giving the users of the package the right to use and distribute
 *    the Package in a more-or-less customary fashion, plus the right to make
 *    reasonable modifications.
 *
 *    Definitions:
 *
 *    "Package" refers to the collection of files distributed by the
 *    Copyright Holder, and derivatives of that collection of files created
 *    through textual modification.
 *
 *    "Standard Version" refers to such a Package if it has not been
 *    modified, or has been modified in accordance with the wishes of the
 *    Copyright Holder.
 *
 *    "Copyright Holder" is whoever is named in the copyright or copyrights
 *    for the package.
 *
 *    "You" is you, if you're thinking about copying or distributing this Package.
 *
 *    "Reasonable copying fee" is whatever you can justify on the basis of
 *    media cost, duplication charges, time of people involved, and so
 *    on. (You will not be required to justify it to the Copyright Holder,
 *    but only to the computing community at large as a market that must bear
 *    the fee.)
 *
 *    "Freely Available" means that no fee is charged for the item itself,
 *    though there may be fees involved in handling the item. It also means
 *    that recipients of the item may redistribute it under the same
 *    conditions they received it.
 *
 *    1. You may make and give away verbatim copies of the source form of the
 *    Standard Version of this Package without restriction, provided that you
 *    duplicate all of the original copyright notices and associated
 *    disclaimers.
 *
 *    2. You may apply bug fixes, portability fixes and other modifications
 *    derived from the Public Domain or from the Copyright Holder. A Package
 *    modified in such a way shall still be considered the Standard Version.
 *
 *    3. You may otherwise modify your copy of this Package in any way,
 *    provided that you insert a prominent notice in each changed file
 *    stating how and when you changed that file, and provided that you do at
 *    least ONE of the following:
 *
 *        a) place your modifications in the Public Domain or otherwise make
 *        them Freely Available, such as by posting said modifications to
 *        Usenet or an equivalent medium, or placing the modifications on a
 *        major archive site such as ftp.uu.net, or by allowing the Copyright
 *        Holder to include your modifications in the Standard Version of the
 *        Package.
 *
 *        b) use the modified Package only within your corporation or
 *        organization.
 *
 *        c) rename any non-standard executables so the names do not conflict
 *        with standard executables, which must also be provided, and provide
 *        a separate manual page for each non-standard executable that
 *        clearly documents how it differs from the Standard Version.
 *
 *        d) make other distribution arrangements with the Copyright Holder.
 *
 *    4. You may distribute the programs of this Package in object code or
 *    executable form, provided that you do at least ONE of the following:
 *
 *        a) distribute a Standard Version of the executables and library
 *        files, together with instructions (in the manual page or
 *        equivalent) on where to get the Standard Version.
 *
 *        b) accompany the distribution with the machine-readable source of
 *        the Package with your modifications.
 *
 *        c) accompany any non-standard executables with their corresponding
 *        Standard Version executables, giving the non-standard executables
 *        non-standard names, and clearly documenting the differences in
 *        manual pages (or equivalent), together with instructions on where
 *        to get the Standard Version.
 *
 *        d) make other distribution arrangements with the Copyright Holder.
 *
 *    5. You may charge a reasonable copying fee for any distribution of this
 *    Package. You may charge any fee you choose for support of this
 *    Package. You may not charge a fee for this Package itself.  However,
 *    you may distribute this Package in aggregate with other (possibly
 *    commercial) programs as part of a larger (possibly commercial) software
 *    distribution provided that you do not advertise this Package as a
 *    product of your own.
 *
 *    6. The scripts and library files supplied as input to or produced as
 *    output from the programs of this Package do not automatically fall
 *    under the copyright of this Package, but belong to whomever generated
 *    them, and may be sold commercially, and may be aggregated with this
 *    Package.
 *
 *    7. C or perl subroutines supplied by you and linked into this Package
 *    shall not be considered part of this Package.
 *
 *    8. The name of the Copyright Holder may not be used to endorse or
 *    promote products derived from this software without specific prior
 *    written permission.
 *
 *    9. THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 *    WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 *    MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
package org.chiba.xml.xforms.ui;

import org.apache.log4j.Category;
import org.chiba.xml.util.DOMUtil;
import org.chiba.xml.xforms.Model;
import org.chiba.xml.xforms.NamespaceCtx;
import org.chiba.xml.xforms.events.EventFactory;
import org.chiba.xml.xforms.exception.XFormsException;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.events.EventTarget;

import java.util.ArrayList;
import java.util.List;

/**
 * Implementation of XForms Select Element.
 *
 * @author <a href="mailto:unl@users.sourceforge.net">Ulrich Nicolas Liss&eacute;</a>
 * @version $Id: Select.java,v 1.32 2004/02/21 01:17:42 joernt Exp $
 */
public class Select extends AbstractFormControl {
    private static final Category LOGGER = Category.getInstance(Select.class);
    private List selected = null;

    /**
     * Creates a new Select object.
     *
     * @param element the DOM Element of this select
     * @param model   the Model this select is bound to
     */
    public Select(Element element, Model model) {
        super(element, model);
    }

    // lifecycle methods

    /**
     * Performs element init.
     *
     * @throws XFormsException if any error occurred during init.
     */
    public void init() throws XFormsException {
        if (getLogger().isDebugEnabled()) {
            getLogger().debug(this + " init");
        }

        initializeInstanceNode();
        initializeDefaultAlert();
        initializeDataElement();
        initializeSelectedItems();
        initializeChildren();
        initializeActions();
    }

    /**
     * Sets the value of this form control.
     * <p/>
     * The bound instance data is updated and the event sequence for this control
     * is executed. Event sequences are described in Chapter 4.6 of XForms 1.0 Recommendation.
     *
     * @param value the value to be set.
     */
    public void setValue(String value) throws XFormsException {
        if (isBound()) {
            this.model.getInstance(getInstanceId()).setNodeValue(getLocationPath(), value);

            updateSelectedItems(); //dispatch of deselect, select events

            dispatchValueChangeSequence();
        }
    }

    /**
     * Performs element update.
     *
     * @throws XFormsException if any error occurred during update.
     */
    public void update() throws XFormsException {
        if (getLogger().isDebugEnabled()) {
            getLogger().debug(this + " update");
        }

        updateDataElement();
        updateSelectedItems();
        updateChildren();
    }

    /**
     * Performs element disposal.
     *
     * @throws XFormsException if any error occurred during disposal.
     */
    public void dispose() throws XFormsException {
        if (getLogger().isDebugEnabled()) {
            getLogger().debug(this + " dispose");
        }

        disposeChildren();
        disposeSelectedItems();
        disposeSelf();
    }

    /**
     * Returns the logger object.
     *
     * @return the logger object.
     */
    protected Category getLogger() {
        return LOGGER;
    }

    // lifecycle template methods

    protected final void initializeSelectedItems() {
        if (isBound()) {
            String instanceValue = " " + getValue() + " ";
            this.selected = new ArrayList();
            NodeList list = this.element.getElementsByTagNameNS(NamespaceCtx.XFORMS_NS, ITEM);

            for (int index = 0; index < list.getLength(); index++) {
                Element item = (Element) list.item(index);
                Element value = (Element) item.getElementsByTagNameNS(NamespaceCtx.XFORMS_NS, VALUE).item(0);

                if (instanceValue.indexOf(" " + DOMUtil.getTextNodeAsString(value) + " ") > -1) {
// select item
                    if (getLogger().isDebugEnabled()) {
                        getLogger().debug(this + " init: selecting item " + item.getAttribute("id"));
                    }

                    item.setAttributeNS(NamespaceCtx.XFORMS_NS,
                            xformsPrefix + ":" + SELECTED_ATTRIBUTE,
                            String.valueOf(true));
                    this.selected.add(item);
                } else {
// deselect item
                    if (getLogger().isDebugEnabled()) {
                        getLogger().debug(this + " init: deselecting item " + item.getAttribute("id"));
                    }

                    item.setAttributeNS(NamespaceCtx.XFORMS_NS,
                            xformsPrefix + ":" + SELECTED_ATTRIBUTE,
                            String.valueOf(false));
                }
            }
        }
    }

    /**
     * updates all selected items and dispatches the appropriate xforms-select and xforms-deselect events.
     */
    protected final void updateSelectedItems() throws XFormsException {
        if (isBound()) {
            String instanceValue = " " + getValue() + " ";
            List toDeselect = this.selected;
            List toSelect = new ArrayList();
            NodeList list = this.element.getElementsByTagNameNS(NamespaceCtx.XFORMS_NS, ITEM);

            for (int index = 0; index < list.getLength(); index++) {
                Element item = (Element) list.item(index);
                Element value = (Element) item.getElementsByTagNameNS(NamespaceCtx.XFORMS_NS, VALUE).item(0);

                if (instanceValue.indexOf(" " + DOMUtil.getTextNodeAsString(value) + " ") > -1) {
                    toSelect.add(item);
                }
            }

            for (int index = 0; index < toDeselect.size(); index++) {
                Element item = (Element) toDeselect.get(index);

                if (!toSelect.contains(item)) {
                    getLogger().debug(this + " update: deselecting item " + item.getAttribute("id"));
                    item.setAttributeNS(NamespaceCtx.XFORMS_NS, xformsPrefix + ":" + SELECTED_ATTRIBUTE,
                            String.valueOf(false));
                    this.container.dispatch((EventTarget) item, EventFactory.DESELECT, null);
                }
            }

            for (int index = 0; index < toSelect.size(); index++) {
                Element item = (Element) toSelect.get(index);

                if (!toDeselect.contains(item)) {
                    getLogger().debug(this + " update: selecting item " + item.getAttribute("id"));
                    item.setAttributeNS(NamespaceCtx.XFORMS_NS, xformsPrefix + ":" + SELECTED_ATTRIBUTE,
                            String.valueOf(true));
                    this.container.dispatch((EventTarget) item, EventFactory.SELECT, null);
                }
            }

            this.selected = toSelect;
        }
    }

    /**
     * frees all resources bound by selected items.
     */
    protected final void disposeSelectedItems() {
        if (isBound()) {
            this.selected.clear();
            this.selected = null;
        }
    }

}
